/********** Create an Application Package 
GRANT CREATE APPLICATION PACKAGE ON ACCOUNT TO ROLE accountadmin;
CREATE APPLICATION PACKAGE test_snowflake_package;
-- SHOW APPLICATION PACKAGES;
USE APPLICATION PACKAGE test_snowflake_package;
CREATE SCHEMA stage_content;
CREATE OR REPLACE STAGE test_snowflake_package.stage_content.hello_snowflake_stage FILE_FORMAT = (TYPE = 'csv' FIELD_DELIMITER = '|' SKIP_HEADER = 1);
USE APPLICATION PACKAGE test_snowflake_package;


CREATE SCHEMA IF NOT EXISTS shared_data;
CREATE TABLE IF NOT EXISTS accounts (ID INT, NAME VARCHAR, VALUE VARCHAR);
INSERT INTO accounts VALUES
  (1, 'Nihar', 'Snowflake'),
  (2, 'Frank', 'Snowflake'),
  (3, 'Benoit', 'Snowflake'),
  (4, 'Steven', 'Acme');

USE APPLICATION PACKAGE mydb;
GRANT USAGE ON SCHEMA public TO SHARE IN APPLICATION PACKAGE mydb;
GRANT SELECT ON TABLE public.sample_product_data TO SHARE IN APPLICATION PACKAGE mydb;

USE APPLICATION PACKAGE test_snowflake_package;
GRANT USAGE ON SCHEMA shared_data TO SHARE IN APPLICATION PACKAGE test_snowflake_package;
GRANT SELECT ON TABLE shared_data.accounts TO SHARE IN APPLICATION PACKAGE test_snowflake_package;

GRANT USAGE ON SCHEMA STAGE_CONTENT TO SHARE IN APPLICATION PACKAGE test_snowflake_package;
GRANT SELECT ON TABLE STAGE_CONTENT.HELLO_SNOWFLAKE_STAGE TO SHARE IN APPLICATION PACKAGE test_snowflake_package;

***************************/
/****************************************************************/
/********** application start *******************/
CREATE APPLICATION ROLE IF NOT EXISTS app_public;
CREATE OR ALTER VERSIONED SCHEMA myschema;
GRANT USAGE ON SCHEMA myschema TO APPLICATION ROLE app_public;

/********** to java spl *******************/
CREATE OR REPLACE PROCEDURE myschema.runSPL(spl VARCHAR )
RETURNS TABLE()
LANGUAGE JAVA
RUNTIME_VERSION = '11'
PACKAGES = ('com.snowflake:snowpark:latest')
IMPORTS = ('/java/esproc-bin-new.jar',
					 '/java/spl-java-func.jar',
					 '/java/bcpkix-jdk18on-1.75.jar',
					 '/java/bcprov-jdk18on-1.75.jar',
					 '/java/snowflake-jdbc-3.14.1.jar',
					 '/java/lz4-1.3.0.jar',
					 '/java/fastjson-1.2.28.jar')
HANDLER = 'com.scudata.lib.snowflake.PlusProcedure.run';

GRANT USAGE ON PROCEDURE myschema.runSPL(VARCHAR) TO APPLICATION ROLE app_public;
/********** to java spl2 *******************/
CREATE OR REPLACE PROCEDURE myschema.runSPL(spl VARCHAR, required_arg variant)
RETURNS TABLE()
LANGUAGE JAVA
RUNTIME_VERSION = '11'
PACKAGES = ('com.snowflake:snowpark:latest')
IMPORTS = ('/java/esproc-bin-new.jar',
					 '/java/spl-java-func.jar',
					 '/java/bcpkix-jdk18on-1.75.jar',
					 '/java/bcprov-jdk18on-1.75.jar',
					 '/java/snowflake-jdbc-3.14.1.jar',
					 '/java/lz4-1.3.0.jar',
					 '/java/fastjson-1.2.28.jar')
HANDLER = 'com.scudata.lib.snowflake.PlusProcedure.run';

GRANT USAGE ON PROCEDURE myschema.runSPL(VARCHAR, variant) TO APPLICATION ROLE app_public;

/********** to java spl3 *******************/
CREATE OR REPLACE function myschema.runUDF(spl VARCHAR)
RETURNS VARIANT
LANGUAGE JAVA
RUNTIME_VERSION = '11'
PACKAGES = ('com.snowflake:snowpark:latest')
IMPORTS = ('/java/esproc-bin-new.jar',
					 '/java/spl-java-func.jar',
					 '/java/bcpkix-jdk18on-1.75.jar',
					 '/java/bcprov-jdk18on-1.75.jar',
					 '/java/snowflake-jdbc-3.14.1.jar',
					 '/java/lz4-1.3.0.jar',
					 '/java/fastjson-1.2.28.jar')
HANDLER = 'com.scudata.lib.snowflake.PlusScudata.calculate';

GRANT USAGE ON function myschema.runUDF(VARCHAR) TO APPLICATION ROLE app_public;

/********** to java spl4 *******************/
CREATE OR REPLACE function myschema.runUDF(spl VARCHAR, argv variant)
RETURNS VARIANT
LANGUAGE JAVA
RUNTIME_VERSION = '11'
PACKAGES = ('com.snowflake:snowpark:latest')
IMPORTS = ('/java/esproc-bin-new.jar',
					 '/java/spl-java-func.jar',
					 '/java/bcpkix-jdk18on-1.75.jar',
					 '/java/bcprov-jdk18on-1.75.jar',
					 '/java/snowflake-jdbc-3.14.1.jar',
					 '/java/lz4-1.3.0.jar',
					 '/java/fastjson-1.2.28.jar')
HANDLER = 'com.scudata.lib.snowflake.PlusScudata.calculate';

GRANT USAGE ON function myschema.runUDF(VARCHAR, variant) TO APPLICATION ROLE app_public;
